/*
  AweMUD - body.h
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#ifndef BODY_H
#define BODY_H

#include <vector>

#include "strings.h"
#include "xml.h"
#include "object.h"

/* GENDER types */
typedef enum gender_spec {
	GENDER_NONE,	// no gender
	GENDER_FEMALE,	// female
	GENDER_MALE,	// male
};

/* BODYLOC types */
typedef enum bodyloc_spec {
	BODYLOC_NONE,
	BODYLOC_HEAD,
	BODYLOC_TORSO,
	BODYLOC_ARMS,
	BODYLOC_LEGS,
	BODYLOC_HANDS,
	BODYLOC_FEET,
	BODYLOC_NECK,
	BODYLOC_BODY,
	BODYLOC_BACK,
	BODYLOC_WAIST,
};

typedef enum bodystatus_spec {
	BODYSTATUS_SEVERED = 1,
	BODYSTATUS_PARA = 2,
	BODYSTATUS_SCARRED = 4,
	BODYSTATUS_INVIS = 8,
};

/* character body */
class char_body
{
	protected:
	int gen;

	object *left_held, *right_held, *body_worn, *back_worn, *waist_worn;

	void do_load (const char *, const xmlNodePtr, object **);
	void do_save (const char *, xmlNodePtr, object *) const;

	public:
	char_body (void);
	~char_body (void);

	int load (const xmlNodePtr);
	void save (xmlNodePtr) const;

	int hold (class object *);
	int wear (class object *);
	int equip (class object *);

	void let_go (class object *);
	void remove (class object *);
	void unequip (class object *);

	bool is_held (object *);
	bool is_worn (object *);
	bool is_equipped (object *);

	int hands_free (void);
	void swap (void);

	void drop_held (class room *);
	void drop_all (class room *);

	class object *get_held (unsigned int);
	class object *get_worn (unsigned int);
	class object *get_equip (unsigned int);

	class object *get_held (string_t);
	class object *get_worn (string_t);
	class object *get_equip (string_t);

	int get_gender (void) const { return gen; }
	void set_gender (int g) { gen = g; }

	void show (class character *);
};

extern const char *body_loc_names[];
extern const char *body_status_names[];
extern const char *body_pos_names[];

#endif
