/*
  AweMUD - callbacks.h
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#ifndef CALLBACKS_H
#define CALLBACKS_H

#include <vector>

class callback_list_0
{
	public:
	typedef void (*funccall)(void);
	typedef vector<funccall> call_list;

	protected:
	vector<funccall> callbacks;

	public:
	void add (funccall func)
	{
		callbacks.push_back (func);
	}
	void remove (funccall func)
	{
		for (call_list::iterator i = callbacks.begin (); i != callbacks.end (); i ++)
			if (*i == func)
			{
				callbacks.erase (i);
				break;
			}
	}
	void call (void)
	{
		for (call_list::iterator i = callbacks.begin (); i != callbacks.end (); i ++)
			(*i)();
	}

	void operator () (void) { call (); }
	
};

template <class C1>
class callback_list_1
{
	public:
	typedef void (*funccall)(C1);
	typedef vector<funccall> call_list;

	protected:
	vector<funccall> callbacks;

	public:
	void add (funccall func)
	{
		callbacks.push_back (func);
	}
	void remove (funccall func)
	{
		for (typename call_list::iterator i = callbacks.begin (); i != callbacks.end (); i ++)
			if (*i == func)
			{
				callbacks.erase (i);
				break;
			}
	}
	void call (C1 c1)
	{
		for (typename call_list::iterator i = callbacks.begin (); i != callbacks.end (); i ++)
			(*i)(c1);
	}

	void operator () (C1 c1) { call (c1); }
	
};

template <class C1, class C2>
class callback_list_2
{
	public:
	typedef void (*funccall)(C1, C2);
	typedef vector<funccall> call_list;

	protected:
	vector<funccall> callbacks;

	public:
	void add (funccall func)
	{
		callbacks.push_back (func);
	}
	void remove (funccall func)
	{
		for (typename call_list::iterator i = callbacks.begin (); i != callbacks.end (); i ++)
			if (*i == func)
			{
				callbacks.erase (i);
				break;
			}
	}
	void call (C1 c1, C2 c2)
	{
		for (typename call_list::iterator i = callbacks.begin (); i != callbacks.end (); i ++)
			(*i)(c1, c2);
	}
	
	void operator () (C1 c1, C2 c2) { call (c1, c2); }
};

template <class C1, class C2, class C3>
class callback_list_3
{
	public:
	typedef void (*funccall)(C1, C2, C3);
	typedef vector<funccall> call_list;

	protected:
	vector<funccall> callbacks;

	public:
	void add (funccall func)
	{
		callbacks.push_back (func);
	}
	void remove (funccall func)
	{
		for (typename call_list::iterator i = callbacks.begin (); i != callbacks.end (); i ++)
			if (*i == func)
			{
				callbacks.erase (i);
				break;
			}
	}
	void call (C1 c1, C2 c2, C3 c3)
	{
		for (typename call_list::iterator i = callbacks.begin (); i != callbacks.end (); i ++)
			(*i)(c1, c2, c3);
	}
	
	void operator () (C1 c1, C2 c2, C3 c3) { call (c1, c2, c3); }
};

template <class C1, class C2, class C3, class C4>
class callback_list_4
{
	public:
	typedef void (*funccall)(C1, C2, C3, C4);
	typedef vector<funccall> call_list;

	protected:
	call_list callbacks;

	public:
	void add (funccall func)
	{
		callbacks.push_back (func);
	}
	void remove (funccall func)
	{
		for (typename call_list::iterator i = callbacks.begin (); i != callbacks.end (); i ++)
			if (*i == func)
			{
				callbacks.erase (i);
				break;
			}
	}
	void call (C1 c1, C2 c2, C3 c3, C4 c4)
	{
		for (typename call_list::iterator i = callbacks.begin (); i != callbacks.end (); i ++)
			(*i)(c1, c2, c3, c4);
	}
	
	void operator () (C1 c1, C2 c2, C3 c3, C4 c4) { call (c1, c2, c3, c4); }
};

#endif
