/*
  AweMUD - command.h
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#ifndef COMMAND_LINE_H
#define COMMAND_LINE_H

#include <vector>

#include "strings.h"
#include "mem.h"
#include "error.h"
#include "char.h"

#define COMM_NA 0
#define COMM_GM 1
#define COMM_ADMIN 2
#define COMM_BUILDER 4
#define COMM_PLAYER 8

namespace commands {
	typedef void (*comm_func) (class character *, char *); /* can manipulate command buffer */
	struct comm_def {
		const char *name;
		unsigned char priv;
		comm_func func;
	};

	char *get_arg (char **); /* updates pointer, returns arg */
	char *get_until (char **, const char **, int* = NULL); /* updates pointer, second is list of words to look for; word found is gotten with another call to get_arg() */
	bool is_arg (char *);
	char *fix_arg (char *, char **); /* give an arg and current pointer, it moves pointer back and unsets the null bit, making it reparsable/complete */

	extern const comm_def commands[];

	int call (class character *, const char *);
}

#endif
