/*
  AweMUD - error.h
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#ifndef ERROR_H
#define ERROR_H

#include <errno.h>
#include <exception>

#define ERR_DEF(d) #d
#define ERR_LINE(line) ERR_DEF (line)

int error (char const *, ...);
int warning (char const *, ...);

#ifndef __FUNCTION__
#define __FUNCTION__
#endif

#define NO_MEM(var,size) error (__FILE__ ", " ERR_LINE (__LINE__) "," ERR_LINE (__FUNCTION__) ": Could not allocate %d bytes for %s", (size), #var)

#ifdef DEBUG
#define return_if_fail(exp) \
	if (!(exp)) { \
		warning ( "File: %s, Line %d (%s): Assertain failed (%s)", \
			__FILE__, \
			__LINE__, \
			__PRETTY_FUNCTION__, \
			#exp \
		); \
		return; \
	}
#define return_val_if_fail(exp,val) \
	if (!(exp)) { \
		warning ( "File: %s, Line %d (%s): Assertain failed (%s)", \
			__FILE__, \
			__LINE__, \
			__PRETTY_FUNCTION__, \
			#exp \
		); \
		return (val); \
	}
#else
#define return_if_fail(exp)
#define return_val_if_fail(exp,val)
#endif

#endif
