/*
  AweMUD - gametime.h
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#ifndef TIME_H
#define TIME_H

#include <vector>

#include "types.h"
#include "strings.h"
#include "xml.h"

#define TICKS_PER_HOUR 900

#define SUN_UP_HOUR 6
#define SUN_DOWN_HOUR 19

class
game_time_def
{
	public:
	struct month_def
	{
		string_t name;
		uint16 day_count;
		uint8 leap_years;	// each leap_years year, month has extra day
	};
	std::vector<month_def> months;
	std::vector<string_t> week_days;

	uint8 get_week_day (const class game_time &) const;
	uint16 days_in_month (const class game_time &) const;

	void load (const xmlNodePtr);
	void save (xmlNodePtr) const;
};

class
game_time
{
	public:
	uint year;
	uint16 month;
	uint16 day;
	uint8 hour;

	uint16 ticks_in_hour;

	game_time (void) { year = 0; month = 0; day = 0; hour = 0; ticks_in_hour = 0; }

	string_t time_str (void) const;
	string_t date_str (void) const;

	bool is_day (void) const { return hour >= SUN_UP_HOUR && hour < SUN_DOWN_HOUR; }
	bool is_night (void) const { return !is_day (); }

	void update (uint);
	void clip_time (void);

	void load (const xmlNodePtr);
	void save (xmlNodePtr) const;
};

#endif
