/*
  AweMUD - object.h
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#ifndef OBJECT_H
#define OBJECT_H

#include <algorithm>

#include "entity.h"
#include "strings.h"
#include "error.h"
#include "error.h"

enum object_contain_type
{
	OBJ_CONT_NONE = 0,
	OBJ_CONT_IN,
	OBJ_CONT_ON,
	OBJ_CONT_UNDER,
	OBJ_CONT_BEHIND,
	OBJ_CONT_UP,
	OBJ_CONT_DOWN,
	OBJ_CONT_COUNT,
};

#define O_FLAG_NONE 0
#define O_FLAG_HIDDEN 1
#define O_FLAG_NO_GET 2

/* object control */
class
object : public entity
{
	E_TYPE(object)

	protected:
	int equip_loc;
	class object *parent;

	class contain_type
	{
		public:
		
		object_contain_type type;
		class entity_list<class object> list;
	};
	vector<contain_type *> container;

	public:
	flags_type object_flags;

	int action (int, entity *, entity *);	// source, target
	int action_count (int) const;

	const char *get_desc (void) const { return desc.c_str (); }
	void set_desc (const char *new_desc) { desc = new_desc; }

	/* save/load */
	virtual void save (xmlNodePtr) const;
	virtual int load (const xmlNodePtr);

	virtual void save (void) const;
	virtual int load (const resource_id &id);

	bool is_hidden (void) const { return CHECK_FLAG (object_flags, O_FLAG_HIDDEN); }
	bool can_get (void) const { return !CHECK_FLAG (object_flags, O_FLAG_NO_GET); }

	void set_hidden (bool v) { SET_FLAG_TO (object_flags, O_FLAG_HIDDEN, v); }
	void set_can_get (bool v) { SET_FLAG_TO (object_flags, O_FLAG_NO_GET, !v); }

	/* stuffs */
	int get_equip_loc (void) const { return equip_loc; }
	void set_equip_loc (int loc) { equip_loc = loc; }

	void update (void);

	/* parent stuff */
	class entity *get_parent (void) const { return parent; }
	void set_parent (class object *);

	class entity_list<object> *get_container (int) const;
	void set_container_exist (int, bool);
		
	object (void);
	~object (void) {}


	bool operator == (const object &o) const { return this != &o; }
};

/* object_list */
typedef entity_list<class object> object_list;

extern const char *object_contain_types[];

#define OBJECT(ent) E_CAST(ent,object)

#endif
