/*
  AweMUD - registry.h
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#ifndef REGISTRY_H
#define REGISTRY_H

#include <vector>

#include "types.h"
#include "xml.h"

/* BASIC REGISTRY THINGY */

template <typename T>
class registry {
	protected:
	typedef std::vector<T *> member_list;
	member_list members ;

	public:
	registry (void) : members () {}
	virtual ~registry (void) {
		for (typename member_list::iterator i = members.begin (); i != members.end (); i ++)
			delete (*i);
	}

	const T *
	find (const char *name) const {
		return_val_if_fail (name, NULL);

		for (typename member_list::const_iterator i = members.begin (); i != members.end (); i ++)
			if (!strcasecmp ((*i)->get_name (), name))
				return *i;
		return NULL;
	}

	T *
	find (const char *name) {
		return_val_if_fail (name != NULL, NULL);

		for (typename member_list::iterator i = members.begin (); i != members.end (); i ++)
			if (!strcasecmp ((*i)->get_name (), name))
				return *i;
		return NULL;
	}

	typedef typename member_list::const_iterator iterator;

	iterator begin (void) const { return members.begin (); }
	iterator end (void) const { return members.end (); }

	void
	add (T *t) {
		return_if_fail (t != NULL);
		members.push_back (t);
	}

	int
	load (const xmlNodePtr info, const char *type) {
		return_val_if_fail (info != NULL, 2);
		return_val_if_fail (type, 1);

		for (xmlNodePtr ch = xml_find_node (info, type); ch != NULL; ch = xml_next_node (ch, type)) {
			T *t = new T ();
			if (!t->load (ch))
				add (t);
			else {
				delete (t);
				return 1;
			}
		}

		return 0;
	}
};


#endif
