/*
  AweMUD - regname.h
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#ifndef __REGNAME_H__
#define __REGNAME_H__

#include <vector>

#include "strings.h"
#include "error.h"

typedef uint32 name_id;

class name_registry
{
	protected:
	typedef vector<string_t> name_list;
	name_list names;

	public:

	name_id operator [] (const char *name) {
		return_val_if_fail (name, 0);

		name_id id = 1;
		for (name_list::const_iterator i = names.begin (); i != names.end (); i ++, id ++)
			if (!strcasecmp (name, i->c_str ()))
				return id;

		names.push_back (name);
		return names.size ();
	}
	string_t operator [] (name_id id) {
		return_val_if_fail (id > 0, (char *)NULL);

		// valid
		if (id > (name_id)names.size ())
			return (char *)NULL;

		return names[id - 1];
	}
};

class const_name_id {
	private:
	name_id id;

	public:
	const_name_id (void) : id (0) {}
	const_name_id (name_id i) : id (i) {}

	void operator = (name_id i) { if (id == 0) id = i; }
	operator name_id (void) { return id; }
};

#endif
