/*
  AweMUD - resource.h
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/


/*
 * NOTE:
 * The resource_mgr class will NOT delete objects once they've been added to
 * the manager.  The manager's purpose is to allow code to find objects
 * based off of an ID.  Any object added to the manager must be publicly
 * descended from the resource class.  It is the program's responsibility
 * to delete objects.  Also, this manager does not deal with the refernce
 * counting scheme the resource class has.  This is used to ensure that all
 * parts of your code work as expected.  Since there is no way for the
 * manager to understand how you want objects to managed, its your problem.
 */

#ifndef RESOURCE_H
#define RESOURCE_H

#include <vector>
#include <list>

#include "types.h"
#include "error.h"
#include "mem.h"
#include "strings.h"

using namespace std;

class resource_id
{
	typedef uint64 h_size;
	union
	{
		struct
		{
			uint32 rand;
			uint16 time;
			uint16 sys;
		} data;
		uint8 bytes[sizeof (h_size) / sizeof (uint8)];
		h_size handle;
	};

	public:
	resource_id (const resource_id &id) { handle = id.handle; }
	resource_id (h_size i) { handle = i; }
	resource_id (void) : handle (0) {};

	void init (void);
	void clear (void);

	uint8 get_hash (void) const;

	bool valid (void) const { return handle != 0; }

	h_size operator = (h_size i) { handle = i; return handle; }
	operator h_size (void) const { return handle; }

	string_t get_handle (void) const;
	void set_handle (const char *);
};

class resource : public referenced
{
	protected:
	resource_id id;

	public:
	resource (void) : referenced () { id = 0; }
	~resource (void) {}

	resource_id get_id (void) const { return id; }
	resource *self (void) { return this; }

	friend class resource_mgr;
};	

#endif
