/*
  AweMUD - select.h
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#ifndef SELECT_H
#define SELECT_H

#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>

#include "error.h"

class select_t
{
	fd_set mset;
	fd_set cset;

	int max_sock;
	public:
	select_t (void)
	{
		max_sock = -1;
		clear ();
	}

	void clear (void) { FD_ZERO (&mset); max_sock = -1; }
	void add (int f) { FD_SET (f, &mset); if (f >= max_sock) max_sock = f + 1; }
	void remove (int f) { FD_CLR (f, &mset); }

	bool check (int f) { return FD_ISSET (f, &cset); }
	void uncheck (int f) { FD_CLR (f, &cset); }

	void do_select (struct timeval b_tv)
	{
		struct timeval tv = b_tv;
		FD_ZERO (&cset);
		if (max_sock < 0) return;
		for (;;)
		{
			cset = mset;
			if (select (max_sock + 1, &cset, NULL, NULL, &tv) < 0)
			{
				if (errno != EINTR)
					error ("select () failed");
			}
			else
				break;
		}
	}
};

/* instantiated in connection.cc */
extern select_t sockets_select;

#endif
