/*
  AweMUD - server.h
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#ifndef SERVER_H
#define SERVER_H

#include <time.h>
#include <vector>

#include "mem.h"
#include "strings.h"
#include "log.h"
#include "world.h"
#include "template.h"
#include "char.h"

#define SERV_PID_FILE "awemud.pid"
#define SERV_PORT 4545
#define SERV_STATUS_PORT 4547

#define MSECS_PER_TICK 200
#define TICKS_PER_SEC (1000 / MSECS_PER_TICK)

#define SIG_SERV_FINISH 1
#define SIG_SERV_ANNOUNCE 2
#define SIG_SERV_BOOT 3
#define SIG_SERV_SAVE_STATE 4

#define AN_NONE 0
#define AN_INDOORS_ONLY 1
#define AN_OUTDOORS_ONLY 2

class server_base
{
	private:
	/* file data */
	string_t pid_file;

	/* user type data */
	string_t user_name;
	string_t group_name;

	/* misc data */
	string_t logon_message_data;
	string_t connect_message_data;
	string_t quit_message_data;
	bool binit;

	/* log file */
	logfile *log;

	/* time */
	struct timeval last_time;
	unsigned long int game_ticks;		// the total number of games ticks that have been processed
	time_t start_time;

	/* the world!! */
	class world *world_data;

	/* entity templates */
	template_list_t obj_templates;
	template_list_t npc_templates;

	public:
	int accept_sock;
	int accept_port;

	int load_data (void);

	string_t logon_message (void) const { return logon_message_data; }
	string_t connect_message (void) const { return connect_message_data; }
	string_t quit_message (void) const { return quit_message_data; }

	server_base (void) : binit (false) {}
	~server_base (void) {}

	void announce (const char *, flags_type = AN_NONE) const;
	void shutdown (void);
	void save (void) const;
	void update (void);

	void log_msg (int, const char *text);
	void log_reset (void);

	/* templates */
	const xmlNodePtr get_obj_template (const char *);
	object *get_obj_template_copy (const char *);
	const xmlNodePtr get_char_template (const char *);
	character *get_char_template_copy (const char *);

	time_t get_start_time (void) const { return start_time; }

	int parse_args (int, char **);
	int parse_conf (const char *);
	int init (int, char **);
	int run (void);
	int finish (void);

	class world *get_world (void) { if (!world_data) { world_data = new world (); world_data->ref (); } return world_data; }
};

extern server_base server;

void init_socket (int);

#endif
