/*
  AweMUD - strings.h
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#ifndef AWE_STRING_H
#define AWE_STRING_H

#include <string.h>
#include <stdarg.h>
#include <time.h>

#include <string>

class string_t : public std::string {
	public:
	string_t (void) : std::string () {}
	string_t (const string_t &st) : std::string (st) {}
	string_t (const char *str) : std::string (str) {}

	operator const char *(void) const { return data() ? c_str () : NULL; }
	bool operator ! (void) const { return length () == 0; }
};

//#include "types.h"
#include "mem.h"
#include "types.h"

#define isvowel (ch) ((ch) == 'a' || (ch) == 'e' || (ch) == 'i' || (ch) == 'o' || (ch) == 'u' || (ch) == 'y')

#if defined(__GNUC__)
#define FORMAT(index,start) __attribute__ ((format (printf, index, start)))
#else
#define FORMAT(i,s)
#endif

/* safe functions - return functions must be explicitly deleted */
char *suprintf (char const *, ...) FORMAT (1,2);
char *vsuprintf (char const *, va_list);

/* special name functions */
const char *get_num_suffix (uint);

/* safe conversion functions */
long satol (const char *);
class string_t timetostr (const time_t &);

/* check functions */
bool str_is_long (const char *);

/* various funcs */
bool str_case_equal (const char *, const char *);

/* match words in a phrase */
int phrase_match (string_t , string_t ); // string to match is first, string being tested is second

/* search text lists */
const char *get_text_of (const char **, unsigned int);
int get_index_of (const char **, const char *);
int guess_index_of (const char **, const char *);
int get_index_of (const char **, const char *, int);
int guess_index_of (const char **, const char *, int);

/* classes */
class strtok_t
{
	protected:
	char *buf;
	char *tok;

	public:
	strtok_t (const char *tokstr) : buf ((char*)NULL), tok (NULL) { if (tokstr) tok = ::strdup (tokstr); }
	~strtok_t (void)
	{
		if (buf)
			free (buf);
		if (tok)
			free (tok);
	}
	const char *last (void) const { return buf; }
	const char *rest (void) const { return tok; }
	const char *strtok (const char *);
	bool more (void) const { return rest () != NULL; }
};

string_t stringf (char const *, ...) FORMAT (1, 2);

#endif
