/*
  AweMUD - world.h
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#ifndef WORLD_H
#define WORLD_H

#include <vector>

#include "room.h"
#include "gametime.h"
#include "log.h"
#include "weather.h"

class world : public area
{
	E_TYPE(world)

	string_t author;
	string_t version;

	string_t starting_room;

	room *rooms;

	weather_region weather;

	public:
	world (void);
	~world (void);

	uchar get_type (void) const { return 3; }

	/* time info */
	game_time_def time_def;
	game_time world_time;
	
	room *get_room (const char *);

	void announce (const char *, flags_type = 0);

	string_t get_author (void) const { return author; }
	void set_author (const char *a) { author = (a); }
	string_t get_version (void) const { return version; }
	void set_version (const char *v) { version = (v); }

	string_t get_weather_desc (void) { return weather.get_cur (); }

	void save_all_players (void) const;

	room *get_start_room (void);

	void update (void);

	void save (xmlNodePtr) const;
	int load (xmlNodePtr);

	void save (void) const;
	int load (void);

	void add_room (room *);
	void del_room (room *);

	void list_rooms (character *) const;

	/* for parsing */
	virtual void parse_char (int, char *buf) const;
};

#define WORLD(ent) E_CAST(ent,world)

#endif
