/*
  AweMUD - char_comm.cc
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#include "server.h"
#include "command.h"
#include "player.h"

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

namespace commands {
	void admin_cshutdown (player *ch, char *) {
		ch->print (CADMIN "Shutdown issued." CNORMAL "\n");
		message (LOG_IMPORTANT, "Shutdown issued by %s\n", ch->get_name ().c_str ());
		server.announce ("Shutting down, NOW!");
		server.save ();
		server.shutdown ();
	}

	void admin_cgrant (player *ch, char *line) {
		char *type = get_arg (&line);
		if (!type || !is_arg (line)) {
			ch->print ("Usage: grant <admin|gm|builder> [player]\n");
			return;
		}

		player *pl = players::find (line);
		if (pl == NULL) {
			ch->print ("Player '%s' is not logged on.\n", line);
			return;
		} else {
			if (!strcasecmp (type, "admin")) {
				if (pl->is_admin ())
					ch->print (CPLAYER "%s" CNORMAL " already has admin privileges.\n", pl->get_name ().c_str ());
				else {
					ch->print ("Admin privileges granted to " CPLAYER "%s" CNORMAL ".\n", pl->get_name ().c_str ());
					pl->print (CADMIN "You have been granted admin privileges." CNORMAL "\n");
					message ("Player %s has been granted admin by %s\n", pl->get_name ().c_str (), ch->get_name ().c_str ());
					pl->grant_admin ();
				}
			} else if (!strcasecmp (type, "gm")) {
				if (pl->is_gm ())
					ch->print (CPLAYER "%s" CNORMAL " already has GM privileges.\n", pl->get_name ().c_str ());
				else {
					ch->print ("GM privileges granted to " CPLAYER "%s" CNORMAL ".\n", pl->get_name ().c_str ());
					pl->print (CADMIN "You have been granted GM privileges." CNORMAL "\n");
					message ("Player %s has been granted GM by %s\n", pl->get_name ().c_str (), ch->get_name ().c_str ());
					pl->grant_gm ();
				}
			} else if (!strcasecmp (type, "builder")) {
				if (pl->is_builder ())
					ch->print (CPLAYER "%s" CNORMAL " already has builder privileges.\n", pl->get_name ().c_str ());
				else {
					ch->print ("Builder privileges granted to " CPLAYER "%s" CNORMAL ".\n", pl->get_name ().c_str ());
					pl->print (CADMIN "You have been granted builder privileges." CNORMAL "\n");
					message ("Player %s has been granted builder by %s\n", pl->get_name ().c_str (), ch->get_name ().c_str ());
					pl->grant_builder ();
				}
			} else
				ch->print ("Unknown privilege set '%s'\n", type);
		}
	}
}
