/*
  AweMUD - gm.cc
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#include "char.h"
#include "error.h"
#include "server.h"
#include "room.h"
#include "world.h"
#include "command.h"
#include "player.h"

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

namespace commands {
	void builder_cunload (player *ch, char *line)
	{
		if (!is_arg (line)) {
			ch->print ("Usage: unload [room]\n");
			return;
		}

		room *r = server.get_world ()->get_room (line);
		if (r) {
			r->save ();
			server.get_world ()->del_room (r);
			ch->print ("Room removed.\n");
			message ("Room %s unloaded by %s\n", line, ch->get_name ().c_str ());
		} else {
			ch->print ("Unknown room '%s'.\n", line);
		}
	}

	void builder_cload (player *ch, char *line) {
		if (!is_arg (line)) {
			ch->print ("Usage: load [room]\n");
			return;
		}

		if (server.get_world ()->get_room (line)) {
			ch->print ("Room '%s' already loaded.\n");
			return;
		}

		xmlDocPtr room_doc = xml_open (stringf ("./world/rooms/%s.room", line));
		if (!room_doc) {
			ch->print ("Couldn't open room '%s'\n", line);
		}

		room *r = new room ();
		r->load (xml_root (room_doc));
		server.get_world ()->add_room (r);
		xml_free (room_doc);
		ch->print ("Loaded room '%s'\n", line);
		message ("Room %s loaded by %s\n", line, ch->get_name ().c_str ());
	}

	void builder_ccreate (player *ch, char *line)
	{
		char *type = get_arg (&line);
		if (!type || !is_arg (line))
		{
			ch->print ("Usage: create [npc|object] <tag>\n");
			return;
		}

		if (!strcasecmp (type, "npc")) {
			character *new_npc = server.get_char_template_copy (line);
			if (!new_npc) {
				ch->print ("Failed to load template.\n");
				return;
			}

			new_npc->enter_room (ROOM (ch->get_par ()), NULL);
			ch->print ("New NPC " CPLAYER "%s" CNORMAL " add.\n", new_npc->get_name ().c_str ());
		} else if (!strcasecmp (type, "object")) {
			object *new_object = server.get_obj_template_copy (line);
			if (!new_object) {
				ch->print ("Failed to load template.\n");
				return;
			}

			ROOM (ch->get_par ())->objects.add (new_object);
			ch->print ("New object " CITEM "%s" CNORMAL " add.\n", new_object->get_name ().c_str ());
		} else {
			ch->print ("I don't know how to make entities of type '%s'.\n", type);
		}
	}
}
