/*
  AweMUD - char_do.cc
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/


#include "char.h"
#include "server.h"
#include "room.h"
#include "world.h"
#include "body.h"
#include "player.h"

void
character::do_emote (char const *action)
{
	if (ROOM (get_par ()))
	{
		char buf[1024];
		snprintf (buf, 1024, "(" CPLAYER "%s" CNORMAL " %s)\n", get_name ().c_str (), action);
		ROOM (get_par ())->print (buf);
	}
}

void
character::do_say (char const *say_what)
{
	if (ROOM (get_par ()))
		ROOM (get_par ())->say (say_what, this);
}

void
character::do_look (void)
{
	if (ROOM (get_par ()))
		ROOM (get_par ())->show (this);
}

void
character::do_look (character *ch)
{
	return_if_fail (ch != NULL);

	print ("%s%s" CNORMAL ":", ch->ncolor (), ch->get_name ().c_str ());
	player *ply;
	if ((ply = PLAYER(ch)) != NULL) {
		print (" %s.\n", ply->body->get_gender () == GENDER_MALE ? "male" : ply->body->get_gender () == GENDER_FEMALE ? "female" : "neuter");
	} else {
		print ("%s\n", ch->get_desc ().c_str ());
	}
	if (ch->is_dead ())
		printf ("%s is dead.\n", ch->get_name ().c_str ());
}

void
character::do_look (const object *obj, int type)
{
	return_if_fail (obj != NULL);

	const object_list *list = NULL;
	if (type != OBJ_CONT_NONE) {
		list = obj->get_container (type);
		if (list) {
			int did_show = 0;
			for (const object_list::item *i = list->entries (); i != NULL; i = i->next) {
				if (!did_show)
				{
					did_show = 1;
					print ("Looking %s the " CITEM "%s" CNORMAL " you see:\n", object_contain_types[type], obj->get_name ().c_str ());
				} else
					print (", ");
				print ("" CITEM "%s" CNORMAL "", i->ent->get_name ().c_str ());
			}
			if (!did_show) {
				print ("There is nothing %s the " CITEM "%s" CNORMAL ".\n", object_contain_types[type], obj->get_name ().c_str ());
			} else {
				print (".\n");
			}

		} else
			print ("" CITEM "%s" CNORMAL " cannot be used like that.\n", obj->get_name ().c_str ());
	} else
		print ("%s\n", obj->get_desc ());
}

void
character::do_look (room_exit *e)
{
	return_if_fail (e != NULL);

	if (e->get_desc ())
		print ("%s" CNORMAL "\n", e->get_desc ().c_str ());
	else
		print ("There is nothing remarkable about " CREXIT "%s" CNORMAL ".\n", e->get_name ().c_str ());

	if (e->is_door ()) {
		if (e->is_closed ()) {
			print ("" CREXIT "%s" CNORMAL " is closed.\n", e->get_name ().c_str ());
		} else {
			print ("" CREXIT "%s" CNORMAL " is open.\n", e->get_name ().c_str ());
		}
	}
}

void
character::do_position (position_spec new_pos)
{
	if (can_act ()) {
		if (new_pos == pos)
		{
			switch (pos)
			{
				case C_SIT:
					print ("You are already sitting down.\n");
					break;
				case C_STAND:
					print ("You are already standing up.\n");
					break;
				case C_LAY:
					print ("You are already laying down.\n");
					break;
				case C_KNEEL:
					print ("You are already kneeling.\n");
					break;
			}
		}
		else
		{
			switch (new_pos)
			{
				case C_SIT:
					rprint (CPLAYER "%s" CNORMAL " sits down.\n", get_name ().c_str ());
					print ("You sit down.\n");
					break;
				case C_STAND:
					rprint (CPLAYER "%s" CNORMAL " stands up.\n", get_name ().c_str ());
					print ("You stand up.\n");
					break;
				case C_LAY:
					rprint (CPLAYER "%s" CNORMAL " lays down.\n", get_name ().c_str ());
					print ("You lay down.\n");
					break;
				case C_KNEEL:
					rprint (CPLAYER "%s" CNORMAL " kneels.\n", get_name ().c_str ());
					print ("You kneel.\n");
					break;
				default:
					rprint (CPLAYER "%s" CNORMAL " shifts position.\n", get_name ().c_str ());
					print ("You shift your position.\n");
					break;
			}
			pos = new_pos;
		}
	} else {
		fail_rt ();
	}
}

void
character::do_get (object *obj, const char *source, object_list *source_list)
{
	return_if_fail (obj != NULL);

	if (can_act ()) {
		if (!body->hands_free ())
			print ("You're hands are full.\n");
		else if (!obj->can_get ())
			print ("You cannot pickup " CITEM "%s" CNORMAL ".\n", obj->get_name ().c_str ());
		else
		{
			obj->ref ();
			obj->do_event ("get", this, NULL);
			if (obj->unref ())
			{
				if (body->hold (obj) < 0)
				{
					if (source)
						print ("You can't get " CITEM "%s" CNORMAL " from %s.\n", obj->get_name ().c_str (), source);
					else
						print ("You can't pick up " CITEM "%s" CNORMAL ".\n", obj->get_name ().c_str ());
				}
				else
				{
					if (source)
					{
						source_list->remove (obj);
						print ("You get " CITEM "%s" CNORMAL " from %s.\n", obj->get_name ().c_str (), source);
						rprint ("" CPLAYER "%s" CNORMAL " gets " CITEM "%s" CNORMAL " from %s.\n", get_name ().c_str (), obj->get_name ().c_str (), source);
					}
					else
					{
						ROOM (get_par ())->objects.remove (obj);
						print ("You pick up " CITEM "%s" CNORMAL ".\n", obj->get_name ().c_str ());
						rprint ("" CPLAYER "%s" CNORMAL " picks up " CITEM "%s" CNORMAL ".\n", get_name ().c_str (), obj->get_name ().c_str ());
					}
				}
			}
		}
	} else {
		fail_rt ("getting any items.");
	}
}

void
character::do_put (object *obj, object *contain, int type)
{
	return_if_fail (obj != NULL);
	return_if_fail (contain != NULL);
	return_if_fail (type != OBJ_CONT_NONE);

	if (can_act ()) {
		if (!obj->can_get ())
			print ("You cannot put " CITEM "%s" CNORMAL " anywhere.\n", obj->get_name ().c_str ());
		else
		{
			if (!contain->get_container (type))
				print ("You cannot do that with " CITEM "%s" CNORMAL ".\n", contain->get_name ().c_str ());
			else
			{
				contain->get_container (type)->add (obj);
				body->let_go (obj);
				print ("You put " CITEM "%s" CNORMAL " %s " CITEM "%s" CNORMAL ".\n", obj->get_name ().c_str (), object_contain_types[type], contain->get_name ().c_str ());
			}
		}
	} else {
		fail_rt ("placing any items.");
	}
}

void
character::do_wear (object *obj)
{
	return_if_fail (obj != NULL);

	if (can_act ()) {
		if (body->wear (obj) < 0)
			print ("You can't wear " CITEM "%s" CNORMAL ".\n", obj->get_name ().c_str ());
		else {
			body->let_go (obj);
			print ("You wear " CITEM "%s" CNORMAL ".\n", obj->get_name ().c_str ());
			show_rt (2 * TICKS_PER_SEC); /* FIXME: depend on armor and char */
			rprint ("" CPLAYER "%s" CNORMAL " equips " CITEM "%s" CNORMAL ".\n", get_name ().c_str (), obj->get_name ().c_str ());
		}
	} else {
		fail_rt ("equipping yourself.");
	}
}

void
character::do_remove (object *obj)
{
	return_if_fail (obj != NULL);

	if (can_act ()) {
		if (!body->hands_free ())
		{
			print ("Your hands are full.\n");
		} else {
			obj->ref ();
			if (body->hold (obj) < 0)
				print ("You can't hold " CITEM "%s" CNORMAL ".\n", obj->get_name ().c_str ());
			else {
				body->remove (obj);
				print ("You remove " CITEM "%s" CNORMAL ".\n", obj->get_name ().c_str ());
				show_rt (2 * TICKS_PER_SEC); /* FIXME: depend on armor and char */
				rprint ("" CPLAYER "%s" CNORMAL " removes " CITEM "%s" CNORMAL ".\n", get_name ().c_str (), obj->get_name ().c_str ());
			}
			obj->unref ();
		}
	} else {
		fail_rt ("unequipping yourself.\n");
	}
}

void
character::do_drop (object *obj)
{
	return_if_fail (obj != NULL);

	if (can_act ()) {
		obj->ref ();
		if (body->is_held (obj))
		{
			print ("You drop " CITEM "%s" CNORMAL ".\n", obj->get_name ().c_str ());
			body->let_go (obj);

			rprint ("" CPLAYER "%s" CNORMAL " drops " CPLAYER "%s" CNORMAL ".\n", get_name ().c_str (), obj->get_name ().c_str ());
			ROOM (get_par ())->objects.add (obj);
		} else
			print ("You are not holding " CITEM "%s" CNORMAL ".\n", obj->get_name ().c_str ());

		obj->unref ();
	} else {
		fail_rt ("dropping anything.\n");
	}
}

void
character::do_read (object *obj)
{
	return_if_fail (obj != NULL);

	if (!can_see ())
	{
		print ("You cannot read anything.\n");
		return;
	}
	if (obj->can_event ("read"))
	{
		print ("You read " CITEM "%s" CNORMAL ".\n", obj->get_name ().c_str ());
		rprint ("" CPLAYER "%s" CNORMAL " reads " CITEM "%s" CNORMAL ".\n", get_name ().c_str (), obj->get_name ().c_str ());
		obj->ref ();
		obj->do_event ("read", this, NULL);
		obj->unref ();
	} else
		print ("You can't read " CITEM "%s" CNORMAL ".\n", obj->get_name ().c_str ());
}

void
character::do_raise (object *obj)
{
	return_if_fail (obj != NULL);

	if (can_act ()) {
		print ("You raise " CITEM "%s" CNORMAL " into the air.\n", obj->get_name ().c_str ());
		rprint ("" CPLAYER "%s" CNORMAL " raises " CITEM "%s" CNORMAL " into the air.\n", get_name ().c_str (), obj->get_name ().c_str ());
		obj->ref ();
		obj->do_event ("raise", this, NULL);
		obj->unref ();
	} else
		fail_rt ();
}

void
character::do_open (room_exit *exit)
{
	return_if_fail (exit != NULL);

	if (can_act ()) {
		if (!exit->is_closed ())
			print ("" CREXIT "%s" CNORMAL " is already open.\n", exit->get_name ().c_str ());
		else if (exit->is_locked ())
			print ("" CREXIT "%s" CNORMAL " is locked.\n", exit->get_name ().c_str ());
		else
		{
			exit->open ();
			print ("You open " CREXIT "%s" CNORMAL ".\n", exit->get_name ().c_str ());
			rprint ("" CPLAYER "%s" CNORMAL " opens " CREXIT "%s" CNORMAL ".\n", get_name ().c_str (), exit->get_name ().c_str ());
		}
	} else
		fail_rt ("opening anything,");
}

void
character::do_close (room_exit *exit)
{
	return_if_fail (exit != NULL);

	if (can_act ()) {
		if (exit->is_closed ())
			print ("" CREXIT "%s" CNORMAL " is already closed.\n", exit->get_name ().c_str ());
		else if (exit->is_locked ())
			print ("" CREXIT "%s" CNORMAL " is locked.\n", exit->get_name ().c_str ());
		else
		{
			exit->close ();
			print ("You close " CREXIT "%s" CNORMAL ".\n", exit->get_name ().c_str ());
			rprint ("" CPLAYER "%s" CNORMAL " closes %s.\n", get_name ().c_str (), exit->get_name ().c_str ());
		}
	} else
		fail_rt ("closing anything.");
}

void
character::do_lock (room_exit *exit)
{
	return_if_fail (exit != NULL);

	if (can_act ()) {
		if (!exit->is_closed ())
			print ("You cannot lock an opened door.\n");
		else if (exit->is_locked ())
			print ("" CREXIT "%s" CNORMAL " is already locked.\n", exit->get_name ().c_str ());
		else
		{
			exit->lock ();
			print ("You lock " CREXIT "%s" CNORMAL ".\n", exit->get_name ().c_str ());
			show_rt (2 * TICKS_PER_SEC);
			rprint ("" CPLAYER "%s" CNORMAL " locks " CREXIT "%s" CNORMAL ".\n", get_name ().c_str (), exit->get_name ().c_str ());
		}
	} else
		fail_rt ("locking anything.");
}

void
character::do_unlock (room_exit *exit)
{
	return_if_fail (exit != NULL);

	if (can_act ()) {
		if (!exit->is_closed ())
			print ("You cannot unlock an opened door.\n");
		else if (!exit->is_locked ())
			print ("" CREXIT "%s" CNORMAL " is already unlocked.\n", exit->get_name ().c_str ());
		else
		{
			show_rt (2 * TICKS_PER_SEC);
			print ("You unlock " CREXIT "%s" CNORMAL ".\n", exit->get_name ().c_str ());
			rprint ("" CPLAYER "%s" CNORMAL " unlocks " CREXIT "%s" CNORMAL ".\n", get_name ().c_str (), exit->get_name ().c_str ());
		}
	} else
		fail_rt ("unlocking anything,");
}

void
character::do_go (const room_exit *exit)
{
	return_if_fail (exit != NULL);

	if (!can_move ())
	{
		if (pos == C_SIT)
			print ("You cannot move while sitting down.\n");
		else if (pos == C_LAY)
			print ("You cannot move while laying down.\n");
		else if (pos == C_KNEEL)
			print ("You cannot move while kneeling.\n");
		else if (is_dead ())
			print ("You cannot move, since you're dead.\n");
		else
			print ("You cannot move.\n");
	}
	else if (can_act ()) 
	{
		if (exit->is_closed ()) {
			print ("" CREXIT "%s" CNORMAL " is closed.\n", exit->get_name ().c_str ());
		} else {
			room *new_room = exit->get_room ();
			if (new_room)
			{
				enter_room (new_room, exit);
				new_room->show (this, true);
			}
			else
				print ("" CREXIT "%s" CNORMAL " does not lead anywhere.\n", exit->get_name ().c_str ());
		}
	} else
		fail_rt ("moving.");
}
