/*
  AweMUD - combat.cc
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/


#include <stdlib.h>

#include "char.h"
#include "object.h"
#include "rand.h"
#include "combat.h"
#include "server.h"
#include "body.h"
#include "command.h"
#include "player.h"
#include "npc.h"

const char *attack_types_pre[] = {
	"slash",
	"smash",
	"stab",
	"thrust",
	"smite",
	"bash",
	"club",
	"beat",
	NULL,
};
const char *attack_types_post[] = {
	"slashed",
	"smashed",
	"stabbed",
	"thrusted",
	"smote",
	"bashed",
	"clubbed",
	"beat",
	NULL,
};
const char *attack_type_present[] = {
	"slashes",
	"smashes",
	"stabs",
	"thrusts",
	"smites",
	"bashes",
	"clubs",
	"beats",
	NULL,
};

int
attack_entity (character *attacker, entity *target)
{
	if (NPC (target) || PLAYER (target))
		return attack_character (attacker, (character *)target);
	else if (OBJECT(target))
		return attack_object (attacker, (object *)target);
	else {
		attacker->print ("You can't attack '%s'.\n", target->get_name ().c_str ());
		return 2;
	}
}

int
attack_object (character *attacker, object *target)
{
	return_val_if_fail (attacker != NULL, 1);

	attacker->print ("" CADMIN "Not implemented yet." CNORMAL "\n");
	return 0;
}


int
attack_character (character *attacker, character *target)
{
	return_val_if_fail (attacker != NULL, 1);
	return_val_if_fail (target != NULL, 1);

	attacker->ref ();
	target->ref ();

	entity_property *com_prop;

	int held = 0;
	int attacks = 0;
	int tdamage = 0;
	object *obj;
	while ((obj = attacker->body->get_held (held ++)) != NULL)
	{
		com_prop = obj->get_property ("attack");
		if (com_prop)
		{
			obj->ref ();

			int attack_mod = com_prop->get_ivalue ("skill") + attacker->get_stat (STAT_STRENGTH) / 10;
			int attack_roll = roll_dice (1, 100);
			int attack = attack_mod + attack_roll;
			int defend = 50;

			attacker->print ("You attack %s%s" CNORMAL "... A:%d + R:%d = %d vs. D:%d -- ", target->ncolor (), target->get_name ().c_str (), attack_mod, attack_roll, attack, defend);
			target->print ("%s%s" CNORMAL " attacks YOU... A:%d + R:%d = %d vs. D:%d -- ", attacker->ncolor (), attacker->get_name ().c_str (), attack_mod, attack_roll, attack, defend);
			if (attack >= defend) {
				int droll = get_random (com_prop->get_ivalue ("dam_range")) + 1;;
				int damage = droll + com_prop->get_ivalue ("dam_mod") + attacker->get_stat (STAT_STRENGTH) / 10;
				if (damage < 1)
					damage = 1;
				attacker->print ("Hit!\n  You deliver %d damage!\n", damage);
				target->print ("Hit!\n  YOU take %d damage!\n", damage);

				tdamage += damage;
			} else {
				attacker->print ("Miss!\n");
				target->print ("Miss!\n");
			}
			
			attacks ++;
			obj->unref ();
		}
	}

	if (!attacks) {
		attacker->print ("You have nothing to attack with.\n");
	} else {
		attacker->show_rt ((3 + 2 * attacks) * TICKS_PER_SEC); /* FIXME: depend on attack skill, encumbrance, etc. */
	}

	target->damage (tdamage);

	target->unref ();
	attacker->unref ();

	return attacks;
}

namespace commands {
	void cattack (character *ch, char *line)
	{
		if (!is_arg (line)) {
			ch->print ("Attack whom?\n");
			return;
		}

		if (ch->can_act ()) {
			character *ach = ROOM (ch->get_par ())->find_char (line);
			if (ach) {
				if (ach == ch) {
					ch->print ("You can't attack yourself.\n");
				} else {
					attack_entity (ch, ach);
				}
			} else {
				ch->print ("You do not see '%s'.\n", line);
			}
		} else {
			ch->fail_rt ("you may attack.");
		}
	}

	void ckill (character *ch, char *line)
	{
		cattack (ch, line);
	}
}
