/*
  AweMUD - emote.cc
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#include "char.h"
#include "emotes.h"
#include "server.h"
#include "xml.h"
#include "parse.h"
#include "command.h"

vector <char_emote> char_emotes;

void
do_emote (character *ch, int emote)
{
	return_if_fail (ch != NULL);
	char buffer[256];
	snprintf (buffer, 256, "%s\n", parse::ereplace (char_emotes[emote].get_action (), ch));
	if (ROOM (ch->get_par ())) {
		ROOM (ch->get_par ())->print (buffer);
	}
}

void
do_emote (character *sch, int emote, const character *tch)
{
	return_if_fail (sch != NULL);
	return_if_fail (tch != NULL);

	char buffer[256];
	snprintf (buffer, 256, "%s\n", parse::ereplace (char_emotes[emote].get_to_action (), sch, tch));

	if (ROOM (sch->get_par ()))
		ROOM (sch->get_par ())->print (buffer);
}

int
handle_emote (character *ch, char *text)
{
	char *arg;
	return_val_if_fail (ch != NULL, 1);

	arg = commands::get_arg (&text);
	if (arg == NULL) {
		return 1;
	}

	int emote = -1;
	int index = 0;
	for (vector<char_emote>::iterator i = char_emotes.begin (); i != char_emotes.end (); i ++) {
		if (!strncasecmp (arg, i->get_name (), strlen (arg))) {
			emote = index;
			break;
		}
		++ index;
	}
	message ("%s %s %d\n", arg, text, index);
	if (emote == -1)
		return 1;

	if (commands::is_arg (text)) {
		character *ech = ROOM (ch->get_par ())->find_char (text);
		if (!ech)
			ch->print ("You do not see '%s'.\n", text);
		else
			do_emote (ch, emote, ech);
	} else
		do_emote (ch, emote);

	return 0;
}

int
init_emotes (void) {
	xmlDocPtr doc = xml_open ("./data/emote");
	if (!doc)
	{
		warning ("Could not load emotes file");
		return 1;
	}
	xmlNodePtr base = xml_root (doc);
	if (strcasecmp ((const char *)base->name, "emotes"))
	{
		xml_free (doc);
		return 1;
	}

	for (xmlNodePtr node = base->children; node != NULL; node = node->next)
	{
		if (!strcasecmp ((const char *)node->name, (const char *)"emote"))
		{
			char_emote e;
			e.set (xml_prop (node, "id"), xml_prop (node, "action"), xml_prop (node, "toaction"), xml_prop (node, "position", 0));
			char_emotes.push_back (e);
		}
	}

	xml_free (doc);

	return 0;
}

int
close_emotes (void) {
	return 0;
}

/* FIXME: make emotes work */
