/*
  AweMUD - file.cc
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <dirent.h>

#include "error.h"
#include "types.h"
#include "file.h"
#include "strings.h"
#include "mem.h"
#include "log.h"

string_t
read_line (const int f, int *eof)
{
	string_t str;
	uchar byte;
	int in;

	if (eof != NULL)
		*eof = 0;

	while ((in = read (f, &byte, 1)) != 0)
	{
		if (in < 0)
		{
			if (errno == EINTR || errno == EAGAIN)
				continue;
			else
			{
				if (eof != NULL)
					*eof = 1;
				break;
			}
		}
		if (byte == '\n') break;
		str += (byte);
	}

	if (in == 0 && eof != NULL)
		*eof = 1;

	return str;
}

bool
does_exist (char const *file_name)
{
	struct stat st;
	return !stat (file_name, &st);
}

bool
is_a_dir (char const *file_name)
{
	struct stat st;
	if (stat (file_name, &st))
		return 0;
	return S_ISDIR (st.st_mode);
}
