/*
  AweMUD - gm.cc
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#include "char.h"
#include "error.h"
#include "server.h"
#include "room.h"
#include "world.h"
#include "command.h"
#include "player.h"

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

namespace commands {
	void gm_cannounce (player *ch, char *line)
	{
		if (!is_arg (line))
			ch->print ("Announce what?\n");
		else
			server.announce (line);
	}

	void gm_cboot (player *ch, char *line)
	{
		if (!is_arg (line)) {
			ch->print ("Boot who?\n");
			return;
		}
		player *cn = players::find(line);
		if (cn) {
			ch->print ("Booting " CPLAYER "%s"  CNORMAL "...\n", cn->get_name ().c_str ());
			cn->print ("" CADMIN "You are being booted by a GM." CNORMAL "\n");
			message ("User %s booted by %s\n", cn->get_name ().c_str (), ch->get_name ().c_str ());
			cn->disconnect ();
		} else
			ch->print ("Player '%s' is not logged in.\n", line);
	}

	void gm_ctoroom (player *ch, char *line)
	{
		if (!is_arg (line)) {
			ch->print ("Jump to which room?\n");
			return;
		}

		room *r = server.get_world ()->get_room (line);
		if (r) {
			ch->print ("Jumping to %s.\n", r->get_name ().c_str ());
			ch->enter_room (r, NULL);
		} else
			ch->print ("Could not find room '%s'.\n", line);
	}
}
