/*
  AweMUD - help.cc
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/


#include <unistd.h>
#include <fcntl.h>

#include "strings.h"
#include "file.h"
#include "server.h"
#include "xml.h"
#include "player.h"

#include "help.h"

xmlDocPtr xml_help_doc;

namespace commands {
	void chelp (player *ch, char *line)
	{
		print_help (ch, line);
	}
}

void
print_help (player *ch, const char *topic)
{
	return_if_fail (ch != NULL);

	if (topic == NULL || topic[0] == '\0')
		topic = "general";

	strtok_t strtok (topic);
	xmlNodePtr node = xml_root (xml_help_doc);

	while (strtok.rest ())
	{
		strtok.strtok (" ");
		node = xml_find_node (node, "topic");
		while (node && !str_case_equal (xml_prop (node, "id"), strtok.last ()))
			node = node->next;
		if (!node)
			break;
	}

	string_t text;
	if (node) {
		node = xml_find_node (node, "body");
		if (node)
			text = xml_text (node);
	}
	if (text.length ())
		ch->print (CSPECIAL "Help for %s:" CNORMAL "\n%s\n---\n", topic, text.c_str ());
	else
		ch->print (CSPECIAL "No help for %s" CNORMAL "\n", topic);
}

int init_help (void)
{
	xml_help_doc = xmlParseFile ("./data/help");

	if (!xml_help_doc)
		return 1;
		
	if (!xml_help_doc->children || !xml_help_doc->children->name || !str_case_equal ((const char *)xml_help_doc->children->name, "help"))
	{
		xmlFreeDoc (xml_help_doc);
		return 1;
	}

	return 0;
}

int close_help (void)
{
	xml_free (xml_help_doc);
	return 0;
}
