/*
  AweMUD - log.cc
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <time.h>

#include "log.h"
#include "error.h"
#include "strings.h"
#include "server.h"

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

logfile::logfile (const char *filename)
{
	return_if_fail (filename);

	f = fopen (filename, "a+");
	file = filename;
	log_level = LOG_MESSAGES;
#ifdef DEBUG
	log_level = LOG_PEDANTIC;
#endif

	fprintf (f, "<--- AweMUD V" VERSION " --->\n");
}

logfile::logfile (FILE *nf, const char *filename)
{
	f = nf;
	file = filename;
	log_level = LOG_MESSAGES;
#ifdef DEBUG
	log_level = LOG_PEDANTIC;
#endif

	fprintf (f, "<--- AweMUD V" VERSION " --->\n");
}

logfile::~logfile (void)
{
	if (f >= 0)
		fclose (f);
}

void
logfile::print (int type, const char *text)
{
	if (!text)
		return;

	if (log_level < type)
		return;

	if (f != NULL)
		fprintf (f, "%s", text);
	else
		puts (text);

}

void
logfile::reset (void)
{
	if (f != NULL) {
		fclose (f);
		f = fopen (file.c_str (), "a+");
		if (f == NULL)
			warning ("Failed to re-load logfile %s", file.c_str ());
	}
}

int
message (char const *err_text, ...)
{
	va_list va;

	char tbuf[41];
	char buf[1024];
	time_t t;
	time (&t);

	strftime (tbuf, 40, "%Y-%m-%d %T", localtime (&t));

	snprintf (buf, 1024, "%s - ", tbuf);
	va_start (va, err_text);
	vsnprintf (buf + strlen (buf), 1024 - strlen (buf), err_text, va);
	va_end (va);

	server.log_msg (LOG_MESSAGES, buf);
	
	return 0;
}

int
message (int level, char const *err_text, ...)
{
	va_list va;

	char tbuf[41];
	char buf[1024];
	time_t t;
	time (&t);

	strftime (tbuf, 40, "%Y-%m-%d %T", localtime (&t));

	snprintf (buf, 1024, "%s - ", tbuf);
	va_start (va, err_text);
	vsnprintf (buf + strlen (buf), 1024 - strlen (buf), err_text, va);
	va_end (va);

	server.log_msg (level, buf);
	
	return 0;
}
