/*
  AweMUD - cstate.cc
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#include <ctype.h>

#include "char.h"
#include "server.h"
#include "file.h"
#include "player.h"
#include "parse.h"

const char *
login_proc::prompt (player *ply)
{
	return state == 0 ? "Enter thy name:" : state == 1 ? "Enter thy passphrase:" : "Create new character?";
}

int
login_proc::process (player *ply, char *line)
{
	switch (state) {
		case 0:
			{
				if (players::find (line)) {
					ply->print (CPLAYER "%s" CNORMAL " is already logged in.\n", line);
				} else if (players::exists (line)) {
					ply->name = line; /* need to set name */
					ply->toggle_echo (false);
					ply->need_update ();
					state = 1;
				} else {
					ply->name = line; /* need to set name */
					ply->need_update ();
					state = 2;
				}
				break;
			}
		case 1:
			{
				ply->toggle_echo (true);

				if (!players::check_pass (ply->get_name (), line))
				{
					ply->print ("\nIncorrect password.\n");
					state = 0;
					break;
				}

				if (players::find (line)) {
					ply->print ("\n" CPLAYER "%s" CNORMAL " already logged in.\n", ply->get_name ().c_str ());
					state = 0;
					break;
				}

				/* lets load the character - initial setup */
				if (ply->load (ply->get_name ())) {
					ply->print ("\n" CADMIN "Error loading player.\n");
					ply->disconnect ();
				} else {
					SET_FLAG(ply->ply_flags, P_FLAG_VALID); /* need to make valid */
					message (LOG_INFORMATION, "%s logged in\n", ply->get_name ().c_str ());
					ply->print ("\n%s\n", parse::ereplace (server.logon_message (), ply, server.get_world ()));
					ply->do_look ();
					return 1;
				}
				break;
			}
		case 2:
			{
				ply->need_update ();
				if (strlen (line) && !strncasecmp (line, "yes", strlen (line))) {
					if (players::find (ply->name) || players::exists (ply->name)) {
						ply->print (CPLAYER "%s" CNORMAL " already exists.\n", ply->name.c_str ());
						state = 1;
					} else {
						ply->add_processor (new create_proc ());
						return 0; /* so as to not delete the new proc */
					}
				} else {
					state = 0;
				}
				break;
			}
		default:
			{
				ply->print (CADMIN "Invalid state, setting to login." CNORMAL "\n");
				state = 0;
				break;
			}
	}

	return 0;
}

const char *
password_proc::prompt (player *)
{
	if (valid) {
		if (new_pass.length ())
			return "Verify password:";
		else
			return "New password:";
	} else {
		return "Current password:";
	}
}

int
password_proc::process (player *ply, char *line)
{
	ply->put ("\n");
	if (valid) {
		if (new_pass.length ()) {
			if (!strcmp (new_pass, line)) {
				players::set_pass (ply->get_name (), line);
				ply->print ("Password changed successfully.\n");
			} else {
				ply->print ("Passwords do not match.\n");
			}
			return true;
		} else {
			ply->need_update ();
			new_pass = line;
		}
	} else {
		if (players::check_pass (ply->get_name (), line)) {
			ply->need_update ();
			valid = true;
		} else {
			ply->print ("Incorrect password.\n");
			return true; /* end */
		}
	}

	return 0;
}

#if 0

void
player::do_changepass (const char *line)
{
	switch (cstate) {
		case CSTATE_OLDPASS:
			{
				if (players::check_pass (name, line)) {
					SET_FLAG (io_flags, CIO_FLAG_NEED_UPDATE);
					cstate == CSTATE_NEWPASS;
				} else {
					print ("Incorrect password.\n");
					cstate = 0;
				}
				break;
			}
		case CSTATE_NEWPASS:
				if (players::find (line)) {
					print (CPLAYER "%s" CNORMAL " is already logged in.\n", line);
				} else if (players::exists (line)) {
					name = line;
					toggle_echo (false);
					cstate = CSTATE_PASS;
					SET_FLAG (io_flags, CIO_FLAG_NEED_UPDATE);
				} else {
					SET_FLAG (io_flags, CIO_FLAG_NEED_UPDATE);
					cstate = CSTATE_ASKCREATE;
				}
int
cstate_processor::init (void)
{
	return_val_if_fail (conn != NULL, 1);
	set_prompt ("Enter thy name:");
	stage = CSTATE_NAME;
	return 0;
}

int
cstate_processor::process (const char *in)
{
	return_val_if_fail (conn != NULL, 1);

	if (stage == CSTATE_NAME) {
		if (!valid_name (in)) {
			conn->put ("Invalid name.\n");
			return 0;
		}

		if (connection::find_player (in)) {
			conn->put (stringf ("%s already logged in.\n", in));
			return 0;
		}

		name = in;

		if (players::exists (name)) {
			conn->toggle_echo (false);
			set_prompt ("Enter thy password:");
			stage = CSTATE_PASS;
			return 0;
		} else {
			char buf[MAX_NAME_LEN + 1];
			uint i;
			buf[0] = toupper (name.at (0));
			for (i = 1; i < name.length () && i < MAX_NAME_LEN; ++ i)
				buf[i] = tolower (name.at (i));
			buf[i] = '\0';
			name = buf;
			set_prompt (stringf ("Create new character '%s'? (y/n):", name.c_str ()));
			stage = CSTATE_CREATE;
			return 0;
		}
	/* we need to check the password */ 
	} else if (stage == CSTATE_PASS) {
		/* enable echo again */
		conn->toggle_echo (true);
		conn->put ("\r\n");

		/* do the password check */
		if (!players::check_pass (name, in))
		{
			conn->put ("Incorrect password.\n");
			stage = CSTATE_NAME;
			set_prompt ("Enter thy name:");
			return 0;
		}

		if (connection::find_player (in)) {
			conn->put (stringf ("%s already logged in.\n", name.c_str ()));
			stage = CSTATE_NAME;
			set_prompt ("Enter thy name:");
			return 0;
		}

		/* lets load the character - initial setup */
		player *ch = players::load (name);

		/* valid ? */
		if (ch) {
			SET_FLAG (ch->ply_flags, P_FLAG_VALID);

			conn->set_character (ch);

			/* logging, signals, etc. */
			message (LOG_INFORMATION, "%s logged in\n", ch->get_name ().c_str ());
			ch->print ("%s\n", parse::ereplace (server.logon_message (), ch, server.get_world ()));
			
			/* clean up */
			ch->do_look ();

		/* need to create */
		} else {
			/* new character */
			ch = new player ();
			ch->set_name (name);
			conn->set_character (ch);

			/* damn */
			warning ("Invalid character %s", name.c_str ());
			conn->put ("Your character seems to be missing.  You must create a new one.\n");
			ch->start_creation ();
		}

		return 1;
	} else if (stage == CSTATE_CREATE) {
		if (in[0] == 'y') {
			if (players::exists (name)) {
				conn->put ("Character already exists.\n");
			} else {
				player *ch = new player ();
				conn->set_character (ch);
				ch->set_name (name);
				ch->start_creation ();
				return 1;
			}
		}

		stage = CSTATE_NAME;
		set_prompt ("Enter thy name:");
	}

	return 0;
}
#endif
