/*
  AweMUD - main.cc
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/


#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <fcntl.h>

#include "error.h"
#include "server.h"
#include "strings.h"
#include "parse.h"

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

int
main (int argc, char **argv)
{
	bool daemon = false;

	/* print out information */
	printf ("AweMUD server V%s, Copyright (C) 2000-2002, AwesomePlay Productions, Inc.\n", VERSION);
	printf ("AweMUD comes with ABSOLUTELY NO WARRANTY; for details see the file COPYING.\n");
	printf ("This is free software, and you are welcome to redistribute it\nunder certain conditions; for details see the file COPYING.\n");

	int success = 1;
	/* load arguments */
	for (int i = 1; i < argc; i ++)
		if (argv[i] != NULL && !strcmp (argv[i], "-D"))
			daemon = true;

	/* daemon fork */
	if (daemon)
	{
		printf ("Forking Daemon...\n");
		if (fork ())
			_exit (0);

		int ttyfd = open ("/dev/tty", O_RDWR);
		ioctl (ttyfd, TIOCNOTTY);
		close (ttyfd);
		setpgid (0, 0);
	}

	/* init server */
	if (server.init (argc, argv))
	{
		fprintf (stderr, "server.init () failed\n");
		return 1;
	}
	success = server.run ();

	return success;
}
