/*
  AweMUD - npc.cc
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#include "body.h"
#include "npc.h"
#include "room.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

npc::npc (void) : character ()
{
	age = 1;
}

npc::~npc (void)
{
}

int
npc::load (const xmlNodePtr info)
{
	age = xml_prop (info, "age", 1);

	level = xml_prop (info, "level", 1);

	return character::load (info);
}

void
npc::save (xmlNodePtr info) const
{
	xml_set_prop (info, "age", age);

	xml_set_prop (info, "level", level);

	character::save (info);
}

int
npc::load (const resource_id &id)
{
	xmlDocPtr char_doc = xml_open (stringf ("world/npcs/%s", id.get_handle ().c_str ()));
	if (!char_doc)
		return 1;

	int ok = load (xml_root (char_doc));

	xml_free (char_doc);

	return ok;
}

void
npc::save (void) const
{
	xmlDocPtr char_doc = xml_create ("character", "http://www.awemud.net/xml/" BASE_VERSION "/player.dtd");
	if (!char_doc)
		return;

	save (xml_root (char_doc));

	xml_save (char_doc, stringf ("world/npcs/%s", id.get_handle().c_str ()));

	xml_free (char_doc);

	return;
}

void
npc::death (void)
{
	do_event ("death", NULL, NULL);
	
	rprint ("%s%s" CNORMAL " has been slain!\n", ncolor (), get_name ().c_str ());

	set_char_flag (C_FLAG_DEAD);
	pos = C_LAY;
	if (body && ROOM(get_par ()))
		body->drop_all (ROOM(get_par ()));
	remove ();
}
