/*
  AweMUD - parse.cc
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#include <ctype.h>

#include "entity.h"
#include "parse.h"
#include "char.h"
#include "server.h"

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#define PARSE_OUT_SIZE 1024

extern const char * per_color (int cur, int max);

namespace parse {
	const char *
	ereplace (const char *in, const entity *ea, const entity *eb) {
		static char out[PARSE_OUT_SIZE];
		char result[PARSE_RESULT_SIZE];
		char *oc = out;

		for (; *in != '\0'; ++ in) {
			if (*in == '%') {
				++ in;
				if (*in == '%') {
					if (oc - out < PARSE_OUT_SIZE - 1) {
						*(oc ++) = '%';
					}
				} else {
					result[0] = '\0';
					if (islower (*in) && ea != NULL) {
						ea->parse_char (*in, result);
					} else if (isupper (*in) && eb != NULL) {
						eb->parse_char (tolower (*in), result);
					}
					if (result[0] != '\0') {
						int len = strlen (result);
						if (len > PARSE_OUT_SIZE - (oc - out) - 1)
							len = PARSE_OUT_SIZE - (oc - out) - 1;
						strncpy (oc, result, len);
						oc += len;
					}
				}
			} else {
				if (oc - out < PARSE_OUT_SIZE - 1) {
					*(oc ++) = *in;
				}
			}
		}

		*oc = '\0';
		return out;
	}

	const char *
	prompt (const char *in, const ::character *ch) {
		static char out[PARSE_OUT_SIZE];
		char result[PARSE_RESULT_SIZE];
		char *oc = out;

		for (; *in != '\0'; ++ in) {
			if (*in == '%') {
				++ in;
				if (*in == '%') {
					if (oc - out < PARSE_OUT_SIZE - 1) {
						*(oc ++) = '%';
					}
				} else {
					result[0] = '\0';
					switch (*in) {
						case 'n':
							snprintf (result, PARSE_RESULT_SIZE, CPLAYER "%s" CNORMAL, ch->get_name ().c_str ());
							break;
						case 'h':
							snprintf (result, PARSE_RESULT_SIZE, "%s%d" CNORMAL, per_color (ch->get_hp (), ch->get_max_hp ()), ch->get_hp ());
							break;
						case 'H':
							snprintf (result, PARSE_RESULT_SIZE, "%s%d" CNORMAL, per_color (ch->get_hp (), ch->get_max_hp ()), ch->get_max_hp ());
							break;
						case 'R':
						{
							snprintf (result, PARSE_RESULT_SIZE, "%u", ch->get_rts ());
							break;
						}
					}
					if (result[0] != '\0') {
						int len = strlen (result);
						if (len > PARSE_OUT_SIZE - (oc - out) - 1)
							len = PARSE_OUT_SIZE - (oc - out) - 1;
						strncpy (oc, result, len);
						oc += len;
					}
				}
			} else {
				if (oc - out < PARSE_OUT_SIZE - 1) {
					*(oc ++) = *in;
				}
			}
		}

		*oc = '\0';
		return out;
	}
}
