/*
  AweMUD - ply_comm.cc
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/


#include "command.h"
#include "player.h"

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

void
player::process_cmd (const char *line)
{
	return_if_fail (line != NULL);

	if (!line)
		return;
	if (line[0] == '#')
		line = last_command;
	else
		last_command = line;

	/* check for talking with \", a special case */
	if (line[0] == '\"' || line[0] == '\'')
	{
		if (line[1] == '\0')
			print ("Say what?\n");
		else
			do_say (line+ 1);
		return;
	}

	/* check for emote'ing with :, a special case */
	if (line[0] == ':' || line[0] == ';')
	{
		if (line[1] == '\0')
			print ("Do what?\n");
		else
			do_emote (line + 1);
		return;
	}

	commands::call (this, line);
}

namespace commands
{
	void ctell (player *ch, char *line)
	{
		if (!is_arg (line)) {
			ch->print ("Tell who?\n");
			return;
		}

		char *name = get_arg (&line);

		if (!is_arg (line)) {
			ch->print ("Tell '%s' what?\n", name);
			return;
		}

		player *cn = players::find (name);
		if (cn) {
			cn->print ("[" CPLAYER "%s" CNORMAL "]: " CTALK "%s" CNORMAL "\n", ch->get_name ().c_str (), line);
			cn->last_tell = ch->get_name ().c_str ();
			ch->print ("Message sent to " CPLAYER "%s" CNORMAL ".\n", cn->get_name ().c_str ());
		}
		else
			ch->print ("No player '%s' found.\n", name);
	}

	void creply (player *ch, char *line)
	{
		if (!ch->last_tell)
		{
			ch->print ("No one has sent you a tell yet.\n");
			return;
		}

		if (!is_arg (line))
		{
			ch->print ("Tell '%s' what?\n", ch->last_tell.c_str ());
			return;
		}

		player *cn = players::find (ch->last_tell);
		if (cn)
		{
			cn->print ("[" CPLAYER "%s" CNORMAL "]: " CTALK "%s" CNORMAL "\n", ch->get_name ().c_str (), line);
			cn->last_tell = ch->get_name ().c_str ();
			ch->print ("Message sent to " CPLAYER "%s" CNORMAL ".\n", cn->get_name ().c_str ());
		}
		else
			ch->print ("No player '%s' found.\n", ch->last_tell.c_str ());
	}

	void cprompt (player *ch, char *line)
	{
		ch->prompt = line;
		if (is_arg (line)) {
			ch->print ("Prompt set to '%s'.\n", line);
		} else {
			ch->print ("Prompt reset to default.\n");
		}
	}

	void cpassword (player *ch, char *)
	{
		ch->print ("Chaning password...\n");
		ch->add_processor (new password_proc ());
	}

	void cinventory (player *ch, char *)
	{
		ch->show_inventory ();
	}

	void cinformation (player *ch, char *)
	{
		ch->self_info ();
	}

	void cstats (player *ch, char *)
	{
		ch->show_stats ();
	}
}
