/*
  AweMUD - resource.cc
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>

#include "resource.h"
#include "rand.h"

void
resource_id::init (void)
{
	static int r_fd = -1;
	struct timeval tv;

	gettimeofday (&tv, NULL);

	if (r_fd == -1) {
		r_fd = open ("/dev/urandom", O_RDONLY | O_NONBLOCK);
		if (r_fd < 0) {
			r_fd = -2;
			data.rand = get_random (0xFFFF);
		} else 
			read (r_fd, &data.rand, sizeof (data.rand));
	} else if (r_fd == -2) {
		data.rand = get_random (0xFFFF);
	} else {
		read (r_fd, &data.rand, sizeof (data.rand));
	}

	data.time = tv.tv_usec & 0xFFFF;
	data.sys = getpid () & 0xFFFF;
}

void
resource_id::clear (void)
{
	data.rand = data.time = data.sys = 0;
}

uint8
resource_id::get_hash (void) const
{
	uint8 hash = 0;
	uint8 byte;

	for (int i = 0; i < 4; i ++)
	{
		byte = *(((char *)&handle) + i);
		if (byte & 1)
			hash |= (1 << i);
	}
	
	return hash;
}

string_t
resource_id::get_handle (void) const
{
	char ret [sizeof (h_size) / sizeof (uint8) * 3];
	snprintf (ret, 3, "%02X", bytes[0]);
	for (uint i = 1; i < sizeof (h_size) / sizeof (uint8); ++ i) {
		snprintf (ret + i * 3 - 1, 4, ":%02X", bytes[i]);
	}
	return ret;
}

void
resource_id::set_handle (const char *str)
{
	return_if_fail (str != NULL);

	char *e;
	for (uint i = 0; i < sizeof (h_size) / sizeof (uint8); ++ i) {
		bytes[i] = strtol (str, &e, 16);
		if (!*e)
			break;
		str += (e - str) + 1;
	}
}
