/*
  AweMUD - weather.cc
  Copyright (c) 2000-2002, AwesomePlay Productions, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.
*/

#include "server.h"
#include "rand.h"
#include "xml.h"
#include "weather.h"

int
weather_type::load (const xmlNodePtr info) {
	title = xml_prop (info, "title");
	desc = xml_find_text (info, "desc");
	upto = xml_find_text (info, "upto");
	downto = xml_find_text (info, "downto");
	chance = xml_prop (info, "chance", 100);
	length = xml_prop (info, "length", 100);

	return 0;
}

int
weather_type::save (xmlNodePtr info) const {
	if (title)
		xml_set_prop (info, "title", title);
	if (desc)
		xml_add_text (info, "desc", desc);
	if (upto) 
		xml_add_text (info, "upto", upto);
	if (downto)
		xml_add_text (info, "downto", downto);
	xml_set_prop (info, "chance", chance);
	xml_set_prop (info, "length", length);

	return 0;
}

int
weather_region::load (const xmlNodePtr info) {
	xmlNodePtr child;

	for (child = xml_find_node (info, "type"); child != NULL; child = xml_next_node (child, "type")) {
		weather_type type;
		type.load (child);
		patterns.push_back (type);
	}

	cur = xml_prop (info, "cur", 0);
	target = xml_prop (info, "target", 0);
	ticks = xml_prop (info, "ticks", 100);

	return 0;
}

int
weather_region::save (xmlNodePtr info) const {
	for (vector<weather_type>::const_iterator i = patterns.begin (); i != patterns.end (); i ++) {
		i->save (xml_add_node (info, "type"));
	}

	xml_set_prop (info, "cur", cur);
	xml_set_prop (info, "target", target);
	xml_set_prop (info, "ticks", ticks);

	return 0;
}

void
weather_region::update (int amount) {
	/* weather logic */
	ticks -= amount;

	if (ticks <= 0)
	{
		int old_cur = cur;
		if (cur < target)
			cur ++;
		else if (cur > target)
			cur --;
		else
		{
			// ugly, but works - hopefully
			int tch = 0;
			unsigned int i;
			for (i = 0; i < patterns.size (); i ++)
				tch += patterns[i].chance;
			int ch = get_random (tch) + 1;
			tch = 0;
			for (i = 0; i < patterns.size (); i ++)
			{
				tch += patterns[i].chance;
				if (tch >= ch)
					break;
			}
			target = i;
		}
		ticks = get_random (patterns[cur].length) * TICKS_PER_HOUR / 4;
		if (cur < old_cur)
			server.get_world ()->announce (stringf ("%s\n", patterns[cur].downto.c_str ()), AN_OUTDOORS_ONLY);
		else if (cur > old_cur)
			server.get_world ()->announce (stringf ("%s\n", patterns[cur].upto.c_str ()), AN_OUTDOORS_ONLY);
	}

	if (ticks % (TICKS_PER_HOUR / 4) && patterns[cur].flavor_texts.size ()) {
		server.get_world ()->announce (stringf ("%s\n", patterns[cur].flavor_texts[get_random (patterns[cur].flavor_texts.size ())].c_str ()), AN_OUTDOORS_ONLY);
	}
}

