#!/bin/sh

if ! which uuencode > /dev/null 2>&1 ; then
	UUENCODE=no
else
	UUENCODE=yes
fi

if [ -z "$1" ] || [ -z "$2" ] ; then
	echo "$0 <out> <in1> [in2] ..."
	exit 1
fi

out="$1"
shift

date=`date`
cat << PACK_END > "$out"
#!/bin/sh
# Archive created on $date

# Usage:
## To extract, just pass the file to /bin/sh:
##  /bin/sh $out
##
## You may also provide a directory to expand to:
##  /bin/sh $out some/dir/to/install/in
PACK_END

cat << 'PACK_END' >> "$out"

# Change directories
if [ -n "$1" ] && [ "$1" != '.' ] ; then
	cd "$1" ||  exit 1
fi
PACK_END

while [ -n "$1" ] ; do
	dir=`dirname "$1"`
	file=`basename "$1"`
	elines=`echo "$1" | sed 's/./-/g'`

	if [ -d "$1" ] ; then
		echo "Ignoring directory $1"
	elif [ -f "$1" ] ; then
		cat << PACK_END >> "$out"

#-------$elines-#
# File: $1 #
#-------$elines-#

# Backup old copy
if [ -f '$1' ] ; then
	mv -f '$1' '$dir/.$file~' || exit 1
fi
PACK_END

		if [ -n "$dir" ] && [ "$dir" != '.' ] ; then
			cat << PACK_END >> "$out"
# Create directory
mkdir -p '$dir' || exit 1
PACK_END
		fi

		if [ "x$UUENCODE" = "xno" ] || file "$1" | grep -q -E "^$1:.*text" ; then
			cat << PACK_END >> "$out"
# Unpack
sed 's/^-//' << 'ARCHIVE_END' > '$1' ||
PACK_END
			sed 's/^/-/' "$1" >> "$out"
		else
			cat << PACK_END >> "$out"
# Unpack
uudecode -o '/dev/stdout' << 'ARCHIVE_END' > '$1' ||
PACK_END
			uuencode "$1" '/dev/stdout' >> "$out"
		fi
		cat << PACK_END >> "$out"
ARCHIVE_END
	exit 1
PACK_END

	fi
	shift
done

cat << PACK_END >> "$out"

# End of archive $out
PACK_END
